/// <reference types="knockout" />
import { Style, StyleObservable } from "./style";
import { Attribute } from "./attribute";
import { IEventListener, IAddEventListenerOptions } from "./eventListener";
import * as types from "./types";
export interface IUiElement {
    render(): HTMLElement;
    getElement(): HTMLElement;
}
export interface UiElementOptions {
    id?: string;
    elementType?: string;
    visible?: boolean | KnockoutObservable<boolean>;
    className?: string | KnockoutObservable<string>;
    attributes?: Array<Attribute>;
    eventListeners?: Array<IEventListener>;
    style?: StyleObservable;
    selectable?: boolean;
    innerText?: string | KnockoutObservable<string>;
    innerTextIsHtml?: boolean;
    addControlToDataDictionary?: boolean;
    children?: Array<IUiElement> | KnockoutObservableArray<IUiElement>;
    mutationObserverCallback?: MutationCallback;
}
export declare abstract class UiElement implements IUiElement {
    protected constructor(tagName: string, elementType: string, options?: UiElementOptions);
    private mutationObserver;
    private style;
    private innerText;
    private innerTextIsHtml;
    private children;
    private display;
    private knockoutSubscriptions;
    private initializeMutationObserver;
    protected visible: boolean;
    protected element: HTMLElement;
    protected readonly options: UiElementOptions;
    protected build(): void;
    protected addSubscription(observable: KnockoutObservable<any>, callback: (newValue: any) => void): void;
    render(): HTMLElement;
    refresh(): void;
    remove(): void;
    handleMessage(message: object): void;
    getElement(): HTMLElement;
    getAttribute(attributeName: types.AttributeName): string | number;
    setAttribute(name: types.AttributeName, value: string | number): void;
    deleteAttribute(attributeName: string): void;
    getStyle(...cssProperties: Array<types.CssProperty>): Style;
    getStyleValue(cssProperty: types.CssProperty): any;
    setStyle(newStyle: Style, overwriteExisting?: boolean): void;
    setStyleNonStandard(name: string, value: any, overwriteExisting?: boolean): void;
    deleteStyle(style: types.CssProperty | Array<types.CssProperty> | Style): void;
    setVisibility(visible: boolean): void;
    addEventListener<TType extends keyof HTMLElementEventMap>(type: TType, listener: (this: UiElement, event: HTMLElementEventMap[TType]) => any, options?: boolean | IAddEventListenerOptions): void;
    getChildren(): Array<IUiElement>;
    setChildren(children: Array<IUiElement>, replace?: boolean): void;
    addChild(newChild: IUiElement, referenceChild?: IUiElement): void;
    removeChild(element: IUiElement): void;
    empty(): void;
    focus(options?: FocusOptions): void;
    focusPreviousSibling(options?: FocusOptions): void;
    focusNextSibling(options?: FocusOptions): void;
    setInnerText(innerText: string): void;
    setInnerHtml(innerHtml: string): void;
    readonly tagName: string;
    readonly attributes: {
        [index: string]: string | number;
    };
}
